package db

import (
	"context"
	"fmt"
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/cmd/sessions/internal"
	"gitea.dwysokinski.me/twhelp/sessions/internal/bundb"
	"gitea.dwysokinski.me/twhelp/sessions/internal/service"
	"github.com/urfave/cli/v2"
	"go.uber.org/zap"
)

const (
	createAPIKeyTimeout = time.Second
)

func newAPIKeyCmd() *cli.Command {
	return &cli.Command{
		Name:  "apikey",
		Usage: "Manages api keys",
		Subcommands: []*cli.Command{
			newCreateAPIKeyCmd(),
		},
	}
}

func newCreateAPIKeyCmd() *cli.Command {
	return &cli.Command{
		Name:  "create",
		Usage: "Creates an API key",
		Action: func(c *cli.Context) error {
			logger := zap.L()

			db, err := internal.NewBunDB()
			if err != nil {
				return fmt.Errorf("internal.NewBunDB: %w", err)
			}
			defer func() {
				_ = db.Close()
			}()

			ctx, cancel := context.WithTimeout(c.Context, createAPIKeyTimeout)
			defer cancel()

			svc := service.NewAPIKey(bundb.NewAPIKey(db), service.NewUser(bundb.NewUser(db)))
			ak, err := svc.Create(ctx, c.String("name"), c.Int64("userId"))
			if err != nil {
				return fmt.Errorf("APIKeyService.Create: %w", err)
			}

			logger.Info("API key created", zap.Int64("id", ak.ID), zap.String("key", ak.Key))

			return nil
		},
		Flags: []cli.Flag{
			&cli.StringFlag{
				Name:     "name",
				Required: true,
			},
			&cli.Int64Flag{
				Name:     "userId",
				Required: true,
			},
		},
	}
}
