package db

import (
	"context"
	"fmt"
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/cmd/sessions/internal"
	"gitea.dwysokinski.me/twhelp/sessions/internal/bundb"
	"gitea.dwysokinski.me/twhelp/sessions/internal/domain"
	"gitea.dwysokinski.me/twhelp/sessions/internal/service"
	"github.com/urfave/cli/v2"
	"go.uber.org/zap"
)

const (
	createUserTimeout = time.Second
)

func newUserCmd() *cli.Command {
	return &cli.Command{
		Name:  "user",
		Usage: "Manages users",
		Subcommands: []*cli.Command{
			newCreateUserCmd(),
		},
	}
}

func newCreateUserCmd() *cli.Command {
	return &cli.Command{
		Name:  "create",
		Usage: "Creates a user",
		Action: func(c *cli.Context) error {
			logger := zap.L()

			db, err := internal.NewBunDB()
			if err != nil {
				return fmt.Errorf("internal.NewBunDB: %w", err)
			}
			defer func() {
				_ = db.Close()
			}()

			params, err := domain.NewCreateUserParams(c.String("name"))
			if err != nil {
				return err
			}

			ctx, cancel := context.WithTimeout(c.Context, createUserTimeout)
			defer cancel()

			user, err := service.NewUser(bundb.NewUser(db)).Create(ctx, params)
			if err != nil {
				return fmt.Errorf("UserService.Create: %w", err)
			}

			logger.Info("user created", zap.Int64("id", user.ID), zap.String("name", user.Name))

			return nil
		},
		Flags: []cli.Flag{
			&cli.StringFlag{
				Name:     "name",
				Required: true,
			},
		},
	}
}
