package model

import (
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/internal/domain"
	"github.com/uptrace/bun"
)

type Session struct {
	bun.BaseModel `bun:"base_model,table:sessions,alias:session"`

	ID        int64     `bun:"id,pk,autoincrement,identity"`
	UserID    int64     `bun:"user_id,nullzero,notnull,unique:sessions_user_id_server_key_key"`
	ServerKey string    `bun:"server_key,type:varchar(10),nullzero,notnull,unique:sessions_user_id_server_key_key"`
	SID       string    `bun:"sid,type:varchar(512),nullzero,notnull"`
	CreatedAt time.Time `bun:"created_at,nullzero,notnull,default:current_timestamp"`
	UpdatedAt time.Time `bun:"updated_at,nullzero,notnull,default:current_timestamp"`
}

func NewSession(p domain.CreateSessionParams) Session {
	return Session{
		UserID:    p.UserID(),
		ServerKey: p.ServerKey(),
		SID:       p.SID(),
		CreatedAt: time.Now(),
		UpdatedAt: time.Now(),
	}
}

func (s Session) ToDomain() domain.Session {
	return domain.Session{
		ID:        s.ID,
		UserID:    s.UserID,
		ServerKey: s.ServerKey,
		SID:       s.SID,
		CreatedAt: s.CreatedAt,
		UpdatedAt: s.UpdatedAt,
	}
}
