package model_test

import (
	"encoding/base64"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/internal/bundb/internal/model"
	"gitea.dwysokinski.me/twhelp/sessions/internal/domain"
	"github.com/google/uuid"
	"github.com/stretchr/testify/assert"
)

func TestSession(t *testing.T) {
	t.Parallel()

	var id int64 = 123
	params, err := domain.NewCreateSessionParams(
		"pl151",
		base64.StdEncoding.EncodeToString([]byte(uuid.NewString())),
		1,
	)
	assert.NoError(t, err)

	result := model.NewSession(params)
	result.ID = id

	apiKey := result.ToDomain()
	assert.Equal(t, id, apiKey.ID)
	assert.Equal(t, params.ServerKey(), apiKey.ServerKey)
	assert.Equal(t, params.SID(), apiKey.SID)
	assert.Equal(t, params.UserID(), apiKey.UserID)
	assert.WithinDuration(t, time.Now(), apiKey.CreatedAt, 10*time.Millisecond)
	assert.WithinDuration(t, time.Now(), apiKey.UpdatedAt, 10*time.Millisecond)
}
