package model

import (
	"strings"
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/internal/domain"
	"github.com/uptrace/bun"
)

type User struct {
	bun.BaseModel `bun:"base_model,table:users,alias:user"`

	ID        int64     `bun:"id,pk,autoincrement,identity"`
	Name      string    `bun:"name,nullzero,type:varchar(255),notnull"`
	NameLower string    `bun:"name_lower,nullzero,type:varchar(255),notnull,unique"`
	CreatedAt time.Time `bun:"created_at,nullzero,notnull,default:current_timestamp"`
}

func NewUser(p domain.CreateUserParams) User {
	return User{
		Name:      p.Name(),
		NameLower: strings.ToLower(p.Name()),
		CreatedAt: time.Now(),
	}
}

func (u User) ToDomain() domain.User {
	return domain.User{
		ID:        u.ID,
		Name:      u.Name,
		CreatedAt: u.CreatedAt,
	}
}
