package model_test

import (
	"encoding/base64"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/sessions/internal/domain"
	"gitea.dwysokinski.me/twhelp/sessions/internal/router/rest/internal/model"
	"github.com/google/uuid"
	"github.com/stretchr/testify/assert"
)

func TestNewSession(t *testing.T) {
	t.Parallel()

	sess := domain.Session{
		ID:        1111,
		UserID:    111,
		ServerKey: "pl151",
		SID:       base64.StdEncoding.EncodeToString([]byte(uuid.NewString())),
		CreatedAt: time.Now(),
		UpdatedAt: time.Now(),
	}
	assertSession(t, sess, model.NewSession(sess))
}

func TestNewGetSessionResp(t *testing.T) {
	t.Parallel()

	sess := domain.Session{
		ID:        1111,
		UserID:    111,
		ServerKey: "pl151",
		SID:       base64.StdEncoding.EncodeToString([]byte(uuid.NewString())),
		CreatedAt: time.Now(),
		UpdatedAt: time.Now(),
	}
	assertSession(t, sess, model.NewGetSessionResp(sess).Data)
}

func assertSession(tb testing.TB, du domain.Session, ru model.Session) {
	tb.Helper()

	assert.Equal(tb, du.ServerKey, ru.ServerKey)
	assert.Equal(tb, du.SID, ru.SID)
}
